program botphone;

{ A little program to drive linphone with a topcom butler 4012

  Copyright (C) 2007 Luca Olivetti <luca@ventoso.org>

  This source is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.

  This code is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.

  A copy of the GNU General Public License is available on the World Wide Web
  at <http://www.gnu.org/copyleft/gpl.html>. You can also obtain it by writing
  to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
  MA 02111-1307, USA.
}

{$mode objfpc}{$H+}

uses
  cthreads,
  Classes, sysutils, BaseUnix, libusb, butler,
  simplelogger, callslist, contactslist, options, unix, dos,
  linphoneprocess;
  

resourcestring
 CALLS_DELETE='Fin, borrar?';
 CALLS_EMPTY='No hay llamadas';
 REDIAL_LAST='Fin llamadas';
 REDIAL_EMPTY='No hay llamadas';
 CONTACTS_EMPTY='No hay contactos';
 CONTACTS_LAST='Fin contactos'; 

//----- event handlers for the phone
type TPhoneHandler=class
  procedure Connected(sender: TObject);
  procedure Disconnected(sender: TObject);
  procedure ContactsKey(sender: TObject; Key:TContactKey); 
  function  GetContactName(number:string):string;
  procedure CallsKey(sender: TObject; Key:TContactKey);
  procedure RedialKey(sender: TObject; Key:TContactKey);
  procedure DialNumber(sender: TObject; number: string);
  procedure DialName(sender: TObject; number: string);
  procedure AcceptCall(sender: TObject);
  procedure Hangup(sender: TObject);
  procedure Digit(sender: TObject; number: string);
  procedure ShowIdKey(sender: TObject; id:string);
end;

//--------- globals
var
  linphonec : TLinphoneProcess;
  phone : TButlerPhone;
  phoneHandler : TPhoneHandler;
  terminated : boolean;
  reload : boolean;
  quitaction,termaction,intaction,usr1action,usr2action,hupaction:PSigActionRec;
  mypid:TPid;
  Calls, Redial:TCalls;
  Contacts : TContacts;
  LastCall : Record
    Call : TCall;
    Wait : Int64;
    Unanswered : boolean;
  end;


procedure GeneralStateChange(state,message:string);
begin
   if terminated then exit;
   if state='GSTATE_POWER_OFF' then
   begin
     Log(LOGLEVEL_INFO,'GSTATE_POWER_OFF');
     Phone.ButlerStatus:=BSClearDisplay;
   end
   else if state='GSTATE_POWER_STARTUP' then
   begin
     Log(LOGLEVEL_INFO,'GSTATE_POWER_STARTUP');
     Phone.ButlerStatus:=BSClearDisplay;
   end
   else if state='GSTATE_POWER_ON' then
   begin
     Log(LOGLEVEL_INFO,'GSTATE_POWER_ON');
     Phone.ButlerStatus:=BSOffline;
   end
   else if state='GSTATE_POWER_SHUTDOWN' then
   begin
     Log(LOGLEVEL_INFO,'GSTATE_POWER_SHUTDOWN');
     Phone.ButlerStatus:=BSClearDisplay;
   end
   else if state='GSTATE_REG_NONE' then
   begin
     Log(LOGLEVEL_INFO,'GSTATE_REG_NONE');
     Phone.ButlerStatus:=BSOffline;
   end
   else if state='GSTATE_REG_OK' then
   begin
     Log(LOGLEVEL_INFO,'GSTATE_REG_OK');
     Phone.ButlerStatus:=BSOnline;
   end
   else if state='GSTATE_CALL_IN_INVITE' then
   begin
     Log(LOGLEVEL_INFO,'GSTATE_CALL_IN_INVITE '+message);
     { I've seen that if I don't answer a call, the same call will
       receive various GSTATE_CALL_IN_INVITE, so the call is registered
       only after a while. Here I check if the call is the same one that's
       still pending to be registered }
     if LastCall.Call<>nil then if LastCall.Call.Number<>message then
     begin
       Calls.Add(LastCall.Call);
       LastCall.Call:=nil;
     end;
     if LastCall.call=nil then LastCall.Call:=TCall.Create(message,now);
     LastCall.Unanswered:=false;
     Phone.Ring(pchar(message));
   end
   else if state='GSTATE_CALL_IN_CONNECTED' then
   begin
     //Call answered, it will not be registered in the lost calls log
     if LastCall.Call<>nil then
     begin
       LastCall.Call.Free;
       LastCall.Call:=nil;
     end;
     LastCall.Unanswered:=false;
     Log(LOGLEVEL_INFO,'GSTATE_CALL_IN_CONNECTED');
   end
   else if state='GSTATE_CALL_END' then
   begin
     Log(LOGLEVEL_INFO,'GSTATE_CALL_END');
     Phone.StopRing;
     Phone.Hangup;
     //starts the time for the unanswered call
     if LastCall.Call<>nil then
     begin
       LastCall.Wait:=GetMsCount;
       LastCall.Unanswered:=true;
     end;
   end
   else if state='GSTATE_CALL_ERROR' then
   begin
     Log(LOGLEVEL_INFO,'GSTATE_CALL_ERROR');
     Phone.StopRing;
     Phone.Hangup;
   end
   else
   begin
     Log(LOGLEVEL_INFO,'Unused/Unknown state '+state+' '+message);
   end;
end;


//------------ butler event handlers

procedure TPhoneHandler.Connected(sender: TObject);
begin
  if linphonec=nil then
    linphonec:=TLinphoneProcess.Create(opt.LinphoneLogFilename, @GeneralStateChange);
end;

procedure TPhoneHandler.Disconnected(sender: TObject);
begin
  if linphonec<>nil then FreeAndNil(linphonec);
end;

procedure TPhoneHandler.ContactsKey(sender: TObject; Key:TContactKey);
var LocContact:TContact;
begin
  case Key of
    CKFirst: LocContact:=Contacts.First;
    CKUp:    LocContact:=Contacts.Prev;
    CKDown:  LocContact:=Contacts.Next;
    else exit;
  end;
  phone.DisplayContact(LocContact.Name);
end;

function TPhoneHandler.GetContactName(number:string):string;
var LocContact:TContact;
begin
  LocContact:=Contacts.FindByNumber(number);
  if LocContact<>nil then result:=LocContact.Name else result:=number;
end;

procedure TPhoneHandler.CallsKey(sender: TObject; Key:TContactKey);
var LocCall:TCall;
begin
  case Key of
    CKFirst: LocCall:=Calls.First;
    CKUp:    LocCall:=Calls.Prev;
    CKDown:  LocCall:=Calls.Next;
    CKExit:
      begin  
        if Calls.Count>0 then Phone.CallsStatus:=CSCalls else Phone.CallsStatus:=CSNoCalls;
        exit;
      end;
    else exit;
  end;
  phone.DisplayCall(GetContactName(LocCall.Number),false,LocCall.DateTime);
end;

procedure TPhoneHandler.RedialKey(sender: TObject; Key:TContactKey);
var LocCall:TCall;
begin
  case Key of
    CKFirst: LocCall:=Redial.First;
    CKUp:    LocCall:=Redial.Prev;
    CKDown:  LocCall:=Redial.Next;
    else exit;
  end;
  phone.DisplayRedial(GetContactName(LocCall.Number),false,LocCall.DateTime);
end;

procedure TPhoneHandler.DialNumber(sender: TObject; number: string);
begin
  Redial.Add(TCall.Create(number,now));
  if linphonec<>nil then linphonec.call(number)
end;

procedure TPhoneHandler.DialName(sender: TObject; number: string);
var LocContact:TContact;
    LocCommand:string;
begin
  if number=CALLS_DELETE then
  begin
    Calls.Clear;
    phone.hangup;
    Phone.CallsStatus:=CSNoCalls;
    exit;
  end; 
  if (number=CALLS_EMPTY) or (number=REDIAL_EMPTY) or (number=REDIAL_LAST) or
     (number=CONTACTS_EMPTY) or (number=CONTACTS_LAST) then
  begin
    Phone.hangup;
    exit;
  end;
  LocContact:=Contacts.FindByName(number);
  if LocContact=nil then
  begin
    //try to invite directly, it may be a missed call or a redial
    if linphonec<>nil then linphonec.call(number);
  end else
  begin
    if (length(LocContact.Number)>=1) and (LocContact.Number[1]='!') then
    begin
      //command
      LocCommand:=Trim(Copy(LocContact.Number,2,Length(LocContact.Number)-1))+' &';
      Log(LOGLEVEL_DEBUG,'executing '+LocCommand);
      Shell(LocCommand);
      Phone.hangup;
      Exit;
    end;
    //normal contact
    if linphonec<>nil then linphonec.call(LocContact.Number);
  end;
end;

procedure TPhoneHandler.AcceptCall(sender: TObject);
begin
  if linphonec<>nil then linphonec.answer;
end;

procedure TPhoneHandler.Hangup(sender: TObject);
begin
  if linphonec<>nil then linphonec.hangup;
end;

procedure TPhoneHandler.Digit(sender: TObject; number: string);
begin
  if (number<>' ') and (number<>'P') and (linphonec<>nil) then linphonec.dtmf(number);
end;

procedure TPhoneHandler.ShowIdKey(sender: TObject; id:string);
var LocContact:TContact;
begin
  LocContact:=Contacts.FindByName(id);
  if LocContact<>nil then Phone.DisplayId(LocContact.Number);
end;

//-------------------------------------------------------------------
// signal handlers

procedure CatchSignal(aSig:Integer);cdecl;
begin
  terminated:=true;
end;

procedure HupSignal(aSig:Integer);cdecl;
begin
  reload:=true;
end;

procedure IncDebugLevel(aSig:Integer);cdecl;
begin
  if LogLevel<high(LogLevel) then
  begin
    LogLevel:=succ(LogLevel);
    log(LogLevel,'Increasing debug level');
  end;
end;

procedure DecDebugLevel(aSig:Integer);cdecl;
begin
  if LogLevel>low(LogLevel) then
  begin
    LogLevel:=pred(LogLevel);
    log(LogLevel,'Decreasing debug level');
  end;
end;

//-------------------------------------------------------------------

begin


{

  If I use fork, the main loop will stop as soon as I start a thread,
  at least under arm, so I don't fork now
  
  mypid:=FpFork;
  if mypid<0 then
  begin
    writeln('fork failed');
    halt(mypid);
  end;
    
  if mypid>0 then Halt(0); //parent
  MainThreadID:=mypid;
  log(LOGLEVEL_INFO, 'forked');
}
  Close(input);
  Close(stderr);
  Assign(stderr,'/dev/null');
  Rewrite(stderr);
  
  if (opt.LogFilename<>'') or opt.UseSyslog then
  begin
    Close(output);
    Assign(output, '/dev/null');
    Rewrite(output);
  end;

  LogLevel:=opt.LogLevel;
  if (opt.LogFilename<>'') and not opt.UseSyslog then SetLogFile(opt.LogFilename);
  UseSyslog:=opt.UseSyslog;

  //-----
  linphonec:=nil;
  //----- create phone
  PhoneHandler:=TPhoneHandler.create;
  Phone:=TButlerPhone.Create;
  Phone.IgnoreChecksumErrors:=opt.IgnoreChecksum;
  if Phone.IgnoreChecksumErrors then log(LOGLEVEL_DEBUG,'ignoring checksum errors');
  Calls:=TCalls.Create(0,false,CALLS_DELETE,CALLS_EMPTY);
  Redial:=TCalls.Create(10,true,REDIAL_LAST,REDIAL_EMPTY);
  Contacts:=TContacts.Create(CONTACTS_LAST,CONTACTS_EMPTY);
  if opt.ContactsFilename<>'' then Contacts.LoadFromFile(opt.ContactsFilename);
  LastCall.Call:=nil;
  LastCall.Unanswered:=false;
  with Phone,PhoneHandler do
  begin
    OnConnected:=@Connected;
    OnDisconnected:=@Disconnected;
    OnDialNumber:=@DialNumber;
    OnDialName:=@DialName;
    OnContactsKey:=@ContactsKey;
    OnCallsKey:=@CallsKey;
    OnRedialKey:=@RedialKey;
    OnShowIdKey:=@ShowIdKey;
    OnAcceptCall:=@AcceptCall;
    OnHangup:=@Hangup;
    OnDigit:=@Digit;
    //there are no calls now, but I may add support for storing them in a file
    if Calls.Count>0 then CallsStatus:=CSCalls else CallsStatus:=CSNoCalls;
    log(LOGLEVEL_INFO,'Starting butler');
  end;

  Phone.Start;
  terminated:=false;
  reload:=false;

  log(LOGLEVEL_INFO,'assign signal handlers');
  new(quitaction);
  with quitaction^ do
  begin
    sa_Handler:=SigActionHandler(@CatchSignal);
    fillchar(sa_mask,sizeof(sa_mask),#0);
    Sa_flags:=0;
    Sa_restorer:=nil;
  end;
  new(termaction);
  with termaction^ do
  begin
    sa_Handler:=SigActionHandler(@CatchSignal);
    fillchar(sa_mask,sizeof(sa_mask),#0);
    Sa_flags:=0;
    Sa_restorer:=nil;
  end;
  new(usr1action);
  with usr1action^ do
  begin
    sa_Handler:=SigActionHandler(@IncDebugLevel);
    fillchar(sa_mask,sizeof(sa_mask),#0);
    Sa_flags:=0;
    Sa_restorer:=nil;
  end;
  new(usr2action);
  with usr2action^ do
  begin
    sa_Handler:=SigActionHandler(@DecDebugLevel);
    fillchar(sa_mask,sizeof(sa_mask),#0);
    Sa_flags:=0;
    Sa_restorer:=nil;
  end;
  new(hupaction);
  with hupaction^ do
  begin
    sa_Handler:=SigActionHandler(@HupSignal);
    fillchar(sa_mask,sizeof(sa_mask),#0);
    Sa_flags:=0;
    Sa_restorer:=nil;
  end;
  new(intaction);
  with intaction^ do
  begin
    sa_Handler:=SigActionHandler(@CatchSignal); //SIG_IGN);
    fillchar(sa_mask,sizeof(sa_mask),#0);
    Sa_flags:=0;
    Sa_restorer:=nil;
  end;
  fpSigAction(SIGQUIT,quitaction,nil);
  fpSigAction(SIGTERM,termaction,nil);
  fpSigAction(SIGINT,intaction,nil);
  fpSigAction(SIGUSR1,usr1action,nil);
  fpSigAction(SIGUSR2,usr2action,nil);
  fpSigAction(SIGHUP,hupaction,nil);

  log(LOGLEVEL_INFO,'entering main loop');
  while not terminated do
  begin
    CheckSynchronize(1000);
    if LastCall.Unanswered then
    begin
      if (GetMsCount-LastCall.Wait)>10000 then
      begin
        Calls.Add(LastCall.Call);
        LastCall.Call:=nil;
        LastCall.Unanswered:=false;
        Phone.CallsStatus:=CSNewCalls;
      end;
    end;
    if reload then
    begin
      reload:=false;
      if opt.ContactsFilename<>'' then Contacts.LoadFromFile(opt.ContactsFilename);
      if not opt.UseSyslog then ReopenLog;
    end;
  end;
  log(LOGLEVEL_INFO,'terminating');
  Phone.Free;
  PhoneHandler.Free;
  if linphonec<>nil then linphonec.free;
  if LastCall.Call<>nil then LastCall.Call.Free;
  Calls.Free;
  Redial.Free;
  Contacts.Free;

end.

