unit callslist;
{ Calls list management for botphone

  Copyright (C) 2007 Luca Olivetti <luca@ventoso.org>

  This source is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.

  This code is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.

  A copy of the GNU General Public License is available on the World Wide Web
  at <http://www.gnu.org/copyleft/gpl.html>. You can also obtain it by writing
  to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
  MA 02111-1307, USA.
}

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils; 


type

  { TCall }

  TCall=class
    Number:string;
    DateTime:TDateTime;
    constructor Create(ANumber:string; ADateTime:TDateTime);
  end;
  
{ TCalls }

TCalls=class
private
  FList: TFpList;
  FLast: TCall;
  FEmpty: TCall;
  FIndex: Integer;
  FMax: Integer;
  FOnTop:boolean;
  function GetCount:integer;
public
  constructor Create(max:integer; ontop:boolean; LastText,EmptyText:string);
  destructor Destroy;override;
  function First:TCall;
  function Next:TCall;
  function Prev:TCall;
  procedure Add(ACall:TCall);
  procedure Clear;
  property Count:integer read GetCount;
end;

implementation


{ TCalls }

constructor TCalls.Create(max: integer; ontop:boolean; LastText,EmptyText:string);
begin
  FMax:=Max;
  FOnTop:=OnTop;
  FList:=TFpList.create;
  FLast:=TCall.Create(LastText,now);
  FEmpty:=TCall.Create(EmptyText,now);
  FIndex:=-1;
end;

destructor TCalls.Destroy;
var i:integer;
begin
  for i:=0 to FList.Count-1 do TCall(FList[i]).Free;
  FList.Free;
  FLast.Free;
  FEmpty.Free;
  inherited Destroy;
end;

function TCalls.First: TCall;
begin
  if FList.Count=0 then
  begin
    result:=FEmpty;
  end else
  begin
    result:=TCall(FList[0]);
    FIndex:=0;
  end;

end;

function TCalls.Next: TCall;
begin
  if FList.Count=0 then
  begin
    Result:=FEmpty;
    exit;
  end;
  if FIndex<FList.Count-1 then
  begin
    FIndex:=Findex+1;
  end else
  if FIndex<FList.Count then
  begin
    FIndex:=Findex+1;
    result:=FLast;
    exit;
  end else
  begin
    FIndex:=0;
  end;
  result:=TCall(FList[FIndex]);

end;

function TCalls.Prev: TCall;
begin

  if FList.Count=0 then
  begin
    Result:=FEmpty;
    exit;
  end;
  
  if Findex>0 then
  begin
    FIndex:=Findex-1;
  end else
  begin
    FIndex:=FList.Count;
    result:=FLast;
    exit;
  end;
  if Findex>FList.Count-1 then FIndex:=FList.Count-1;
  Result:=TCall(FList[FIndex]);
end;

procedure TCalls.Add(ACall: TCall);
var i:integer;
begin
  if not Fontop or (FList.Count=0) then FList.Add(ACall)
                                   else FList.Insert(0,ACall);
  if (FMax<=0) or (FList.count<FMax) then exit;
  if Fontop then i:=FList.Count-1 else i:=0;
  TCall(FList[i]).Free;
  Flist.Delete(i);

end;

function TCalls.GetCount:integer;
begin
  Result:=FList.Count;
end;

procedure TCalls.Clear;
var i:integer;
begin
  for i:=0 to FList.Count-1 do TCall(FList[i]).free;
  FList.Clear;
end;

{ TCall }

constructor TCall.Create(ANumber: string; ADateTime: TDateTime);
begin
  Number:=ANumber;
  DateTime:=ADateTime;
end;


end.

