unit contactslist;
{ Contacts list for botphone

  Copyright (C) 2007 Luca Olivetti <luca@ventoso.org>

  This source is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.

  This code is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.

  A copy of the GNU General Public License is available on the World Wide Web
  at <http://www.gnu.org/copyleft/gpl.html>. You can also obtain it by writing
  to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
  MA 02111-1307, USA.
}

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, simplelogger;


type

  { TContact }

  TContact=class
    Name,Number:string;
    online:boolean;
    constructor Create(AName,ANumber:string);
  end;
  
{ TContacts }

TContacts=class
private
  FNames: TStringList;
  FNumbers: TStringList;
  FLast: TContact;
  FEmpty: TContact;
  FIndex: Integer;
  function GetCount:integer;
public
  constructor Create(LastText,EmptyText:string);
  destructor Destroy;override;
  function First:TContact;
  function Next:TContact;
  function Prev:TContact;
  function FindByName(Name:string):TContact;
  function FindByNumber(Number:string):TContact;
  procedure LoadFromFile(Filename:string);
  procedure Add(AContact:TContact);
  procedure Clear;
  property Count:integer read GetCount;
  
end;

implementation


{ TContacts }

constructor TContacts.Create(LastText,EmptyText:string);
begin
  FNames:=TStringList.create;
  FNumbers:=TStringList.create;
  FLast:=TContact.Create(LastText,'');
  FEmpty:=TContact.Create(EmptyText,'');
  FIndex:=-1;
end;

destructor TContacts.Destroy;
var i:integer;
begin
  for i:=0 to FNames.Count-1 do TContact(FNames.Objects[i]).Free;
  FNames.Free;
  FNumbers.Free;
  inherited Destroy;
end;

function TContacts.First: TContact;
begin
  if FNames.Count=0 then
  begin
    result:=FEmpty;
  end else
  begin
    result:=TContact(FNames.Objects[0]);
    FIndex:=0;
  end;

end;

function TContacts.Next: TContact;
begin
  if FNames.Count=0 then
  begin
    Result:=FEmpty;
    exit;
  end;
  if FIndex<FNames.Count-1 then
  begin
    FIndex:=Findex+1;
  end else
  if FIndex<FNames.Count then
  begin
    FIndex:=Findex+1;
    result:=FLast;
    exit;
  end else
  begin
    FIndex:=0;
  end;
  result:=TContact(FNames.Objects[FIndex]);

end;

function TContacts.Prev: TContact;
begin

  if FNames.Count=0 then
  begin
    Result:=FEmpty;
    exit;
  end;
  
  if Findex>0 then
  begin
    FIndex:=Findex-1;
  end else
  begin
    FIndex:=FNames.Count;
    result:=FLast;
    exit;
  end;
  if Findex>FNames.Count-1 then FIndex:=FNames.Count-1;
  Result:=TContact(FNames.Objects[FIndex]);
end;

procedure TContacts.Add(AContact: TContact);
begin
  FNames.AddObject(AContact.name,AContact);
  FNumbers.AddObject(AContact.number,AContact);
end;

function TContacts.GetCount:integer;
begin
  Result:=FNames.Count;
end;

procedure TContacts.Clear;
var i:integer;
begin
  for i:=0 to FNames.Count-1 do TContact(FNames.Objects[i]).free;
  FNames.Clear;
  FNumbers.Clear;
end;

function TContacts.FindByName(Name:string):TContact;
var
  p:integer;
begin
  p:=FNames.IndexOf(Name);
  if p>=0 then Result:=TContact(FNames.Objects[p]) else result:=nil;
end;  

function TContacts.FindByNumber(Number:string):TContact;
var
  p:integer;
begin
  p:=FNumbers.IndexOf(Number);
  if p>=0 then Result:=TContact(FNumbers.Objects[p]) else result:=nil;
end;  

procedure TContacts.LoadFromFile(Filename:string);
var f:textfile;
    p:integer;
    line:string;
begin
  if not FileExists(Filename) then
  begin
   Log(LOGLEVEL_ERROR,'Contacts filename '+Filename+' doesn''t exist');
   exit;
  end;
  assignfile(f,filename);
  {$I-}
  reset(f);
  {$I+}
  if IOResult<>0 then
  begin
    Log(LOGLEVEL_ERROR, 'Error opening '+Filename);
    exit;
  end;
  clear;
  while not eof(f) do
  begin
    readln(f,line);
    p:=pos(',',line);
    if p>0 then Add(TContact.Create(trim(copy(line,1,p-1)),trim(copy(line,p+1,length(line)-p))));
  end;
  closefile(f);
end;
{ TContact }

constructor TContact.Create(AName,ANumber: string);
begin
  Name:=AName;
  Number:=ANumber;
end;


end.

