unit example;

{$mode objfpc}{$H+}

{ Test program for the usermode driver for the Topcom Butler 4012 USB voip phone

  Copyright (C) 2007 Luca Olivetti <luca@ventoso.org>

  This source is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.

  This code is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.

  A copy of the GNU General Public License is available on the World Wide Web
  at <http://www.gnu.org/copyleft/gpl.html>. You can also obtain it by writing
  to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
  MA 02111-1307, USA.
}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs,
  StdCtrls, ExtCtrls, Buttons, Butler;

type

  { TExampleForm }

  TExampleForm = class(TForm)
    SilentRingButton: TButton;
    Contacts: TMemo;
    Calls: TMemo;
    Label3: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Redial: TMemo;
    Label1: TLabel;
    Label2: TLabel;
    Label4: TLabel;
    RingButton: TButton;
    StopRingButton: TButton;
    HangupButton: TButton;
    StatusComboBox: TComboBox;
    CallsComboBox: TComboBox;
    CallerIdEdit: TEdit;
    Events: TMemo;
    procedure RingButtonClick(Sender: TObject);
    procedure SilentRingButtonClick(Sender: TObject);
    procedure StopRingButtonClick(Sender: TObject);
    procedure HangupButtonClick(Sender: TObject);
    procedure StatusComboBoxChange(Sender: TObject);
    procedure CallsComboBoxChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { private declarations }
    phone:TButlerPhone;
    procedure PConnected(sender:TObject);
    procedure PDisconnected(sender:TObject);
    procedure PDialNumber(sender:TObject;number:string);
    procedure PDialName(sender:TObject;number:string);
    procedure PAcceptCall(sender:TObject);
    procedure PHangup(sender:TObject);
    procedure PDigit(sender:TObject;number:string);
    procedure PContactsKey(sender:TObject;key:TContactKey);
    procedure PCallsKey(sender:TObject;key:TContactKey);
    procedure PRedialKey(sender:TObject;key:TContactKey);
    procedure PShowIdKey(sender:TObject;n:string);
  public
    { public declarations }
  end; 

var
  ExampleForm: TExampleForm;

implementation

const KeyName:array[TContactKey] of string=('First','Up', 'Down','Exit');

{ TExampleForm }
procedure TExampleForm.PConnected(sender: TObject);
begin
  Events.Lines.Add('connected');
end;

procedure TExampleForm.PDisconnected(sender: TObject);
begin
  Events.Lines.Add('disconnected');
end;

procedure TExampleForm.PContactsKey(sender: TObject;Key:TContactKey);
begin
  Events.Lines.Add('Contacts key: '+KeyName[Key]);
  case Key of
    CKFirst: Contacts.Tag:=0;
    CKUp: if Contacts.Tag>0 then Contacts.Tag:=Contacts.Tag-1;
    CKDown: if Contacts.Tag<Contacts.Lines.Count-1 then Contacts.Tag:=Contacts.Tag+1;
  end;
  Phone.DisplayContact(Contacts.Lines[Contacts.Tag]);
end;

procedure TExampleForm.PCallsKey(sender: TObject; key: TContactKey);
begin
  Events.Lines.Add('Calls key: '+KeyName[Key]);
  case Key of
    CKFirst: Calls.Tag:=0;
    CKUp: if Calls.Tag>0 then Calls.Tag:=Calls.Tag-1;
    CKDown: if Calls.Tag<Calls.Lines.Count-1 then Calls.Tag:=Calls.Tag+1;
  end;
  Phone.DisplayCall(Calls.Lines[Calls.Tag],true,now);
end;

procedure TExampleForm.PRedialKey(sender: TObject; key: TContactKey);
begin
  Events.Lines.Add('Redial key: '+KeyName[Key]);
  case Key of
    CKFirst: Redial.Tag:=0;
    CKUp: if Redial.Tag>0 then Redial.Tag:=Redial.Tag-1;
    CKDown: if Redial.Tag<Redial.Lines.Count-1 then Redial.Tag:=Redial.Tag+1;
  end;
  Phone.DisplayRedial(Redial.Lines[Redial.Tag],false,now);
end;

procedure TExampleForm.PDialNumber(sender: TObject; number: string);
begin
  Events.Lines.Add('dialled number: '+number);
end;

procedure TExampleForm.PDialName(sender: TObject; number: string);
begin
  Events.Lines.Add('dialled name: '+number);
  phone.hangup; //a real program should lookup the name
end;

procedure TExampleForm.PAcceptCall(sender: TObject);
begin
  Events.Lines.Add('call accepted');
end;

procedure TExampleForm.PHangup(sender: TObject);
begin
  Events.Lines.Add('hanged up');
end;

procedure TExampleForm.PDigit(sender: TObject; number: string);
begin
  Events.Lines.Add('dialled digit: '+number);
end;


procedure TExampleForm.PShowIdKey(sender: TObject; n: string);
begin
  Events.Lines.Add('requested id of: '+n);
  Phone.DisplayId('ID '+n);
end;


procedure TExampleForm.FormCreate(Sender: TObject);
begin
  Phone:=TButlerPhone.Create;
  with Phone do
  begin
    OnConnected:=@PConnected;
    OnDisconnected:=@PDisconnected;
    OnCallsKey:=@PCallsKey;
    OnContactsKey:=@PContactsKey;
    OnRedialKey:=@PRedialKey;
    OnShowIdKey:=@PShowIdKey;
    OnDialNumber:=@PDialNumber;
    OnDialName:=@PDialName;
    OnAcceptCall:=@PAcceptCall;
    OnHangup:=@PHangup;
    OnDigit:=@PDigit;
  end;
end;

procedure TExampleForm.RingButtonClick(Sender: TObject);
begin
  Phone.Ring(CalleridEdit.Text);
end;

procedure TExampleForm.SilentRingButtonClick(Sender: TObject);
begin
  Phone.SilentRing(CalleridEdit.Text);
end;

procedure TExampleForm.StopRingButtonClick(Sender: TObject);
begin
  phone.StopRing;
end;

procedure TExampleForm.HangupButtonClick(Sender: TObject);
begin
  Phone.Hangup;
end;

procedure TExampleForm.StatusComboBoxChange(Sender: TObject);
var
  LocStatus:TButlerStatus;
begin
  if StatusComboBox.ItemIndex<=ord(BSLogOff) then LocStatus:=TButlerStatus(StatusComboBox.ItemIndex) else
  begin
    Case StatusComboBox.Itemindex-ord(BSLogOff) of
      1:LocStatus:=BSClearDisplay;
      2:LocStatus:=BSMSN;
      3:LocStatus:=BSSjPhone;
      4:LocStatus:=BSNet2Phone;
    end;
  end;
  Phone.ButlerStatus:=LocStatus;
end;

procedure TExampleForm.CallsComboBoxChange(Sender: TObject);
begin
  Phone.CallsStatus:=TCallsStatus(CallsComboBox.ItemIndex);
end;


procedure TExampleForm.FormDestroy(Sender: TObject);
begin
  Phone.Free;
end;

procedure TExampleForm.FormShow(Sender: TObject);
begin
  Phone.Start;
end;

initialization
  {$I example.lrs}

end.

