unit linphoneprocess;
{ linphonec process controller

  note that for this class to work, linphone has to be patched to
  output status messages.linphoneprocess

  Copyright (C) 2007 Luca Olivetti <luca@ventoso.org>

  This source is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.

  This code is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.

  A copy of the GNU General Public License is available on the World Wide Web
  at <http://www.gnu.org/copyleft/gpl.html>. You can also obtain it by writing
  to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
  MA 02111-1307, USA.
}

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Process, baseunix, unix, termio, simplelogger;
  
type

{ TLinphoneProcess }
TStatusChangeProc = procedure (status,message:string);
TLinphoneProcess = class(TThread)
private
  FProcess:TProcess;
  FBuffer:string;
  FStatus,FMessage:string;
  FStatusChangeProc:TStatusChangeProc;
  procedure Execute;override;
  procedure Decode;
  procedure StatusChange;
  procedure Send(s:string);
public
  constructor Create(logfilename:string;scp:TStatusChangeProc);
  destructor Destroy;override;
  procedure call(number:string);
  procedure hangup;
  procedure answer;
  procedure dtmf(number:string);
end;

implementation

{ TLinphoneProcess }

constructor TLinphoneProcess.Create(logfilename:string;scp:TStatusChangeProc);
begin
  FStatusChangeProc:=scp;
  FProcess:=TProcess.Create(nil);
  FProcess.commandline:='linphonec -S';
  log(LOGLEVEL_DEBUG,'about to execute '+FProcess.commandline);
  if logfilename<>'' then FProcess.CommandLine:=FProcess.CommandLine+' -d 1 -l '+logfilename;
  FProcess.options:=[poUsePipes];
  try
    FProcess.execute;
  except
    on e:exception do log(LOGLEVEL_ERROR,'Error executing linphonec: '+e.message);  
  end;
  inherited Create(false);
end;

destructor TLinphoneProcess.Destroy;
begin
  terminate;
  Send('quit');
  sleep(1000);
  FProcess.WaitOnExit;
  FProcess.Free;
  Log(LOGLEVEL_DEBUG,'Linphonec terminated');
  inherited Destroy;
end;

procedure TLinphoneProcess.Execute;
var c:char;
begin 
  if not FProcess.Running then exit;
  while not terminated do
  begin
     try
       c:=chr(FProcess.output.ReadByte);
       if c=#10 then
       begin
         Decode;
         if FBuffer='GSTATE_POWER_SHUTDOWN' then exit;
         FBuffer:='';
       end else FBuffer:=FBuffer+c;
     except
       on e:exception do log(LOGLEVEL_ERROR,'Error reading from linphonec: '+e.message);
     end;  
  end;
end;

procedure TLinphoneProcess.Decode;
var p:integer;
begin
  log(LOGLEVEL_DEBUG,'Line received from linphonec: '+FBuffer);
  while copy(FBuffer,1,11)='linphonec> ' do delete(FBuffer,1,11);
  if copy(FBuffer,1,7)<>'GSTATE_' then exit;
  p:=pos(' ',FBuffer);
  if p>0 then
  begin
    FStatus:=copy(FBuffer,1,p-1);
    FMessage:=copy(FBuffer,p+1,length(FBuffer));
  end else
  begin
    FStatus:=FBuffer;
    FMessage:='';
  end;
  Synchronize(@StatusChange);
end;


procedure TLinphoneProcess.StatusChange;
begin
  if assigned(FStatusChangeProc) then FStatusChangeProc(FStatus,FMessage);
end;


procedure TLinphoneProcess.Send(s: string);
  var i:integer;
begin
  if not FProcess.Running then exit;
  log(LOGLEVEL_DEBUG,'Sending to linphonec: '+s);
  try
    for i:=1 to length(s) do FProcess.input.writebyte(ord(s[i]));
    FProcess.input.writebyte(10);
  except
    on e:exception do log(LOGLEVEL_ERROR,'Error writing to linphonec: '+e.message);
  end;  
end;

procedure TLinphoneProcess.call(number:string);
begin
  send('call '+number);
end;

procedure TLinphoneProcess.hangup;
begin
  send('terminate');
end;

procedure TLinphoneProcess.answer;
begin
  send('answer');
end;

procedure TLinphoneProcess.dtmf(number:string);
begin
  send(number);
end;

end.

