unit options;
{ Command line options for botphone

  Copyright (C) 2007 Luca Olivetti <luca@ventoso.org>

  This source is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.

  This code is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.

  A copy of the GNU General Public License is available on the World Wide Web
  at <http://www.gnu.org/copyleft/gpl.html>. You can also obtain it by writing
  to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
  MA 02111-1307, USA.
}

{$mode objfpc}{$H+}

interface

uses getopts,simplelogger;

type 
  TOpt=record
    LinphoneConfig:string;
    LogFilename:string;
    LogLevel:TLogLevel;
    UseSyslog:boolean;
    IgnoreChecksum:boolean;
    LinphoneLogFilename:string;
    ContactsFilename:string;
  end; 
  
var

  opt:TOpt;
  
implementation

procedure readoptions;

var c: char;
    optionindex : Longint;
    theopts : array[1..7] of TOption;
    errors : boolean;
begin
  errors:=false;
  with opt do
  begin
    LinphoneConfig:='';
    LogFilename:='';
    LogLevel:=LOGLEVEL_ERROR;
    LinphoneLogFilename:='';
    IgnoreChecksum:=false;
  end;
  with theopts[1] do
  begin
    name:='verbose';
    has_arg:=0;
    flag:=nil;
    value:='v';
  end;
  with theopts[2] do
  begin
    name:='syslog';
    has_arg:=0;
    flag:=nil;
    value:='s';
  end;
  with theopts[3] do
  begin
    name:='logfile';
    has_arg:=1;
    flag:=nil;
    value:='l';
  end;
  with theopts[4] do
  begin
    name:='linphonelogfile';
    has_arg:=1;
    flag:=nil;
    value:='L';
  end;
  with theopts[5] do
  begin
    name:='help';
    has_arg:=0;
    flag:=nil;
    value:='h';
  end;
  with theopts[6] do
  begin
    name:='contacts';
    has_arg:=1;
    flag:=nil;
    value:='C';
  end;
  with theopts[7] do
  begin
    name:='ignore-checksum';
    has_arg:=0;
    flag:=nil;
    value:='i';
  end;
  c:=#0;
  repeat
    c:=getlongopts('vsl:L:hC:i',@theopts[1],optionindex);
    case c of
      'v':if opt.LogLevel<LOGLEVEL_DEBUG then opt.LogLevel:=opt.LogLevel+1;
      's':opt.UseSyslog:=true;
      'l':opt.Logfilename:=optarg;
      'L':opt.LinphoneLogFilename:=optarg;
      'C':opt.ContactsFilename:=optarg;
      'i':opt.IgnoreChecksum:=true;
      '?','h':errors:=true;
    end; //case    
  until c=endofoptions;
  if optind<=paramcount then
  begin
    //writeln('garbage at end of options');
    //errors:=true;
  end; 
  if errors then
  begin
    writeln;
    writeln('usage:'                                                    );
    writeln;
    writeln(argv[0],' options'                                          );
    writeln;
    writeln('available options:'                                        );
    writeln;
    writeln('--contacts|-C <filename>'                                  );
    writeln('   contacts file (simple text file with <name>,<number> on');
    writeln('   each line)'                                             );
    writeln('--verbose|-v'                                              );
    writeln('   verbose, repeat the option to increases verbosity'      );
    writeln('--logfile|-l <filename>'                                   );
    writeln('   log to <filename> instead of stdout'                    );
    writeln('--syslog|-s'                                               );
    writeln('   log to syslog instead of stdout or file'                );
    writeln('--ignore-checksum|-i'                                      );
    writeln('   ignore wrong checksum received from the phone'          );
    writeln('--linphonelogfile|-L <filename>'                           );
    writeln('   enable linphonec log to file'                           );
    writeln('--help|-h'                                                 );
    writeln('   this help message');
    halt(1); 
  end;
end;    

initialization

readoptions;
end.
