unit simplelogger;

{$mode objfpc}{$H+}

 { Simple logging functions

  Copyright (C) 2007 Luca Olivetti <luca@ventoso.org>

  This source is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.

  This code is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.

  A copy of the GNU General Public License is available on the World Wide Web
  at <http://www.gnu.org/copyleft/gpl.html>. You can also obtain it by writing
  to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
  MA 02111-1307, USA.
}

interface

uses
  Classes, SysUtils, systemlog, syncobjs, BaseUnix;

const
  LOGLEVEL_NONE = 0;
  LOGLEVEL_ERROR = 1;
  LOGLEVEL_INFO = 2;
  LOGLEVEL_DEBUG = 3;

type
  TLogLevel=LOGLEVEL_NONE..LOGLEVEL_DEBUG;
var

  LogLevel : TLogLevel;
  UseSyslog : boolean;
  procedure SetLogFile(filename:string);
  procedure ReopenLog;
  procedure Log(level:integer; message:string);
  procedure LogHex(level:integer; message,hexmessage:string);
  

implementation

var
  LogFile:text;
  LogAccess:TCriticalSection;
  
  procedure SetLogFile(filename:string);
  begin
    LogAccess.Acquire;
    close(LogFile);
    assign(LogFile,filename);
    rewrite(LogFile);
    LogAccess.Release;
  end;
  
  procedure ReopenLog;
  begin
    LogAccess.Acquire;
    close(LogFile);
    rewrite(LogFile);
    LogAccess.Release;
  end;
  
  procedure Log(Level:integer; message:string);
  begin
    if LogLevel<Level then exit;
    if UseSyslog then syslog(LOG_INFO,'%s',[Pchar(message)]) else
    begin
      LogAccess.Acquire;
      Writeln(LogFile,message);
      Flush(LogFile);
      LogAccess.Release;
    end;
  end;

  function HexString(s:string):string;
  var i:Integer;
  begin
    result:='';
    for i:=1 to length(s) do result:=result+inttohex(ord(s[i]),2)+' ';
  end;

  procedure LogHex(Level: integer; message,hexmessage:string);
  var
    hexs:string;
  begin
    if LogLevel<Level then exit;
    hexs:=HexString(hexmessage);
    if UseSyslog then syslog(LOG_INFO, '%s %s', [Pchar(message),PChar(hexs)]) else
    begin
      LogAccess.Acquire;
      Writeln(LogFile,message,' ',hexs);
      Flush(LogFile);
      LogAccess.Release;
    end;
  end;

initialization
  LogAccess:=TCriticalSection.Create;
  LogLevel:=LOGLEVEL_NONE;
  Assign(LogFile,'');
  Rewrite(LogFile);
  UseSyslog:=false;

finalization
  LogAccess.Free;
  close(LogFile);

end.

